// =============================================================================
//	Copyright (C) 2009-2014, Heartland.Data Inc. All Rights Reserved.
//	
//	Title:	4bit GPIO Driver
//	
// =============================================================================

// =============================================================================
// 	Please customize the code for your environment.
// =============================================================================

//------------------------------------------------------------------------------
//	Desc:	Header for Port Controll
//------------------------------------------------------------------------------
#include "Common.h"

//------------------------------------------------------------------------------
//  Macro:	DT_UINT
//	Desc:	Please change Test Point argument type for DT10 Project setting.
//------------------------------------------------------------------------------
#define	DT_UINT unsigned int

//------------------------------------------------------------------------------
//	Macro:	DT_INLINE
//	Desc: 	Please use "static" instead of "inline" if "inline" cannot be used.
//------------------------------------------------------------------------------
#define	DT_INLINE	inline
//#define	DT_INLINE	static

//------------------------------------------------------------------------------
//  Macro:	DT_ADD_TIME_INFO
//	Desc:	Please set 1 if use TARGET SYSTEM TIME information with Test Point.
//------------------------------------------------------------------------------
#define DT_ADD_TIME_INFO 0

//------------------------------------------------------------------------------
//  Macro:	DT_WRITE_VARIABLE
//	Desc:	Please set 1 if use Write Test Point
//------------------------------------------------------------------------------
#define DT_WRITE_VARIABLE 0

//------------------------------------------------------------------------------
//	Func:	_TP_BusPortInit
//	Desc: 	Please describe the code to initializes ports.
//------------------------------------------------------------------------------
static void _TP_BusPortInit(void)
{
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_2 );		// CLK
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_3 );		// CS
	GPIOPinTypeGPIOInput( GPIO_PORTD_BASE, GPIO_PIN_6 );		// SEL
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_2 );		// DAT0
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_3 );		// DAT1
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_4 );		// DAT2
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_5 );		// DAT3
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2);
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, GPIO_PIN_3);
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, GPIO_PIN_2 );
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_3, GPIO_PIN_3 );
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_4, GPIO_PIN_4 );
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_5, GPIO_PIN_5 );
}

//------------------------------------------------------------------------------
//	Func:	portReverseCLK
//	Desc:	Please describe the code to reverse CLK Port.
//------------------------------------------------------------------------------
DT_INLINE void portReverseCLK(void)
{
	if( GPIOPinRead( GPIO_PORTD_BASE, GPIO_PIN_2 ) ){
		GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, 0 );
	}
	else{
		GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2 );
	}
}

//------------------------------------------------------------------------------
//	Macro:	portSetCLK
//	Desc:	Please describe the code to set the value to CLK Port.
//------------------------------------------------------------------------------
DT_INLINE void portSetCLK( DT_UINT dat)
{
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

//------------------------------------------------------------------------------
//	Func:	portSetCS
//	Desc:	Please describe the code to set the value to CS Port.
//------------------------------------------------------------------------------
DT_INLINE void portSetCS(DT_UINT dat)
{
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, dat?0xff:0x00 );
}

//------------------------------------------------------------------------------
//	Func:	portSetDATA
//	Desc:	Please describe the code to set the value to DATA Port
//------------------------------------------------------------------------------
DT_INLINE void portSetDATA(DT_UINT dat)
{
	GPIOPinWrite( GPIO_PORTA_BASE, (GPIO_PIN_2|GPIO_PIN_3|GPIO_PIN_4|GPIO_PIN_5), dat << 2 );
}

#if DT_WRITE_VARIABLE
//------------------------------------------------------------------------------
//	Func:	portGetDATA
//	Desc:	Please describe the code to get the value from SEL Port
//------------------------------------------------------------------------------
DT_INLINE DT_UINT portGetSEL()
{
	return GPIOPinRead( GPIO_PORTD_BASE, GPIO_PIN_6 );
}
#endif

//------------------------------------------------------------------------------
//	Func:	enterCritical
//	Desc:	Please describe the code to enter Critical Section.
//------------------------------------------------------------------------------
DT_INLINE void enterCritical(void)
{
	taskENTER_CRITICAL();
}

//------------------------------------------------------------------------------
//	Func:	exitCritical
//	Desc:	Please describe the code to exit Critical Section.
//------------------------------------------------------------------------------
DT_INLINE void exitCritical(void)
{
	taskEXIT_CRITICAL();
}

#if DT_ADD_TIME_INFO
//------------------------------------------------------------------------------
//	Func:	_TP_GetTargetTime(void)
//	Desc:	
//------------------------------------------------------------------------------
DT_INLINE unsigned int _TP_GetTargetTime(void)
{
	unsigned int time = 0;
	time = TimerValueGet(TIMER0_BASE, TIMER_A);
	return time;	
}
#endif

// =============================================================================
// 	Don't change the code from here as possible.
// =============================================================================

//------------------------------------------------------------------------------
//	Desc:	Test Point IDs
//------------------------------------------------------------------------------
#define		DT_VARIABLE_BIT			0x02
#define		DT_TIME_INFO_BIT		0x04
#define		DT_EVTTRG_BIT			0x08
#define		DT_VARIABLE_FAST_BIT	0x01
#define		DT_EVTTRG_FAST_BIT		0x02
#define		DT_VARIABLE_WRITE_BIT	0x01

//------------------------------------------------------------------------------
//	Func:	portInit
//	Desc:	Initialize Port Function
//------------------------------------------------------------------------------
static int init = 0;
DT_INLINE void portInit(void)
{
	if( init == 0 ){
		_TP_BusPortInit();
		init = 1;
	}
}

//------------------------------------------------------------------------------
//	Func: _TP_Bus4BitOutDrv
//	Desc: 4bit Data Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_Bus4BitOutDrv( DT_UINT dat )
{
	portSetDATA( dat );
	portReverseCLK();
}

//------------------------------------------------------------------------------
//	Func: _TP_BusOutDrv
//	Desc: Test Point Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	portSetCS(0);
	_TP_Bus4BitOutDrv( dat >> 12 );
	_TP_Bus4BitOutDrv( dat >> 8 );
	_TP_Bus4BitOutDrv( dat >> 4 );
	_TP_Bus4BitOutDrv( dat );
	addr &= 0xfffff;
	if( addr >= 0x10000 ) _TP_Bus4BitOutDrv( addr >> 16 );
	if( addr >= 0x1000 ) _TP_Bus4BitOutDrv( addr >> 12 );
	if( addr >= 0x0100 ) _TP_Bus4BitOutDrv( addr >> 8 );
	if( addr >= 0x0010 ) _TP_Bus4BitOutDrv( addr >> 4 );
	_TP_Bus4BitOutDrv( addr );
	portSetCS(1);
}

//------------------------------------------------------------------------------
//	Func:	_TP_BusOutByteDrv
//	Desc:	Byte Data Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_BusOutByteDrv( DT_UINT dat )
{
	_TP_Bus4BitOutDrv( dat >> 4 );
	_TP_Bus4BitOutDrv( dat );
}

//------------------------------------------------------------------------------
//	Func:	_TP_BusOutFastDrv
//	Desc:	Test Point Output(Fast) Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_BusOutFastDrv( DT_UINT step, DT_UINT bit )
{
	portSetCS(0);
	if( bit >= 9 ) _TP_Bus4BitOutDrv( step >> 8 );
	if( bit >= 5 ) _TP_Bus4BitOutDrv( step >> 4 );
	_TP_Bus4BitOutDrv( step );
	portSetCS(1);
}

#if DT_ADD_TIME_INFO
//------------------------------------------------------------------------------
//	Func:	_TP_TimeInfoOutDrv(void)
//	Desc:	Time Info Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_TimeInfoOutDrv(void)
{
	unsigned int time = _TP_GetTargetTime();

	portSetCS(0);
	_TP_BusOutByteDrv( time );
	_TP_BusOutByteDrv( time >> 8);
	_TP_BusOutByteDrv( time >> 16);
	_TP_BusOutByteDrv( time >> 24);
	portSetCS(1);
}
#endif

//------------------------------------------------------------------------------
//	Func:	_TP_MemoryOutDrv
//	Desc:	Value Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_MemoryOutDrv( unsigned char *p, DT_UINT size )
{
#if DT_ADD_TIME_INFO
	unsigned int time = _TP_GetTargetTime();
#endif
	portSetCS(0);
#if DT_ADD_TIME_INFO
	_TP_BusOutByteDrv( time );
	_TP_BusOutByteDrv( time >> 8);
	_TP_BusOutByteDrv( time >> 16);
	_TP_BusOutByteDrv( time >> 24);
#endif
	if( size >= 256 ) size = 256;
	_TP_BusOutByteDrv( size );
	for( ; size != 0; --size, ++p ){
		_TP_BusOutByteDrv( *p );
	}
	portSetCS(1);
}

#if DT_WRITE_VARIABLE
//------------------------------------------------------------------------------
//	Func:	_TP_InputByteDrv
//	Desc:	Byte Data Input Function
//------------------------------------------------------------------------------
DT_INLINE DT_UINT _TP_InputByteDrv( void )
{
	DT_UINT	i, c;
	c = 0;
	for( i = 0; i < 8; i++ ){
		portSetCLK(0);
		c <<= 1;
		if( portGetSEL() ) c |= 0x01;
		portSetCLK(1);
	}
	return c;
}

//------------------------------------------------------------------------------
//	Func:	_TP_WritePointDrv
//	Desc:	Value Write Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_WritePointDrv( unsigned char *p, DT_UINT size )
{
	int	i, s;
	for( i = 0; i < 50; i++ ){
		portSetCLK(0);
		s = portGetSEL();
		portSetCLK(1);
		if( s ) break;
	}
	if( s == 0 ) return ;
	s = _TP_InputByteDrv();
	if( s > 16 || s == 0 ) return;
	for( i = 0; i < s; i++, p++ ){
		*p = _TP_InputByteDrv();
	}
}
#endif

//------------------------------------------------------------------------------
//	Func: _TP_EventOutDrv
//	Desc: Event Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_EventOutDrv( DT_UINT dat )
{
	portSetCS(0);
	_TP_BusOutByteDrv( (dat) );
	_TP_BusOutByteDrv( (dat) >> 8 );
	_TP_BusOutByteDrv( 0x00 );
	_TP_BusOutByteDrv( 0x00 );
	portSetCS(1);
}

//------------------------------------------------------------------------------
//	Func: _TP_BusOut
//	Desc: Called by Test Point
//------------------------------------------------------------------------------
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
#if DT_ADD_TIME_INFO
	addr |= DT_TIME_INFO_BIT;
#endif
	portInit();
	enterCritical();
	_TP_BusOutDrv( addr, dat );
#if DT_ADD_TIME_INFO
	_TP_TimeInfoOutDrv();
#endif
	exitCritical();
}

//------------------------------------------------------------------------------
//	Func: _TP_MemoryOutput
//	Desc: Called by Variable Test Point
//------------------------------------------------------------------------------
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
#if DT_ADD_TIME_INFO
	addr |= DT_VARIABLE_BIT | DT_TIME_INFO_BIT;
#else
	addr |= DT_VARIABLE_BIT;
#endif
	portInit();
	enterCritical();
	_TP_BusOutDrv( addr, dat );
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	exitCritical();
}

//------------------------------------------------------------------------------
//	Func: _TP_EventTrigger
//	Desc: Called by Event Trigger Test Point
//------------------------------------------------------------------------------
void _TP_EventTrigger( DT_UINT addr, DT_UINT dat )
{
#if DT_ADD_TIME_INFO
	addr |= DT_EVTTRG_BIT |DT_TIME_INFO_BIT;
#else
	addr |= DT_EVTTRG_BIT;
#endif
	portInit();
	enterCritical();
	_TP_BusOutDrv( addr, ((unsigned short)(dat&0x0FFF))<<4 );
#if DT_ADD_TIME_INFO
	_TP_TimeInfoOutDrv();
#endif
	exitCritical();
}

#if DT_WRITE_VARIABLE
//------------------------------------------------------------------------------
//	Func: _TP_WritePoint
//	Desc: Called by Variable Write Test Point
//------------------------------------------------------------------------------
void _TP_WritePoint( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
#if DT_ADD_TIME_INFO
	addr |= DT_VARIABLE_WRITE_BIT | DT_TIME_INFO_BIT;
#else
	addr |= DT_VARIABLE_WRITE_BIT;
#endif
	portInit();
	enterCritical();
	_TP_BusOutDrv( addr, dat );
#if DT_ADD_TIME_INFO
	_TP_TimeInfoOutDrv();
#endif
	_TP_WritePointDrv( value, size );
	exitCritical();
}
#endif

//------------------------------------------------------------------------------
//	Func: _TP_BusOutFast
//	Desc: Called by Fast Test Point
//------------------------------------------------------------------------------
void _TP_BusOutFast( DT_UINT step, DT_UINT bit )
{
	portInit();
	step = step << 2;
	bit = bit + 2;
	enterCritical();
	_TP_BusOutFastDrv( step, bit );
	exitCritical();
}

//------------------------------------------------------------------------------
//	Func: _TP_MemoryOutputFast
//	Desc: Called by Variable Fast Test Point
//------------------------------------------------------------------------------
void _TP_MemoryOutputFast( DT_UINT step, DT_UINT bit, void *value, DT_UINT size )
{
	portInit();
	step = (step << 2) + DT_VARIABLE_FAST_BIT;
	bit = bit + 2;
	enterCritical();
	_TP_BusOutFastDrv( step, bit );
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	exitCritical();
}

//------------------------------------------------------------------------------
//	Func: _TP_EventTriggerFast
//	Desc: Called by Event Trigger Fast Test Point
//------------------------------------------------------------------------------
void _TP_EventTriggerFast( DT_UINT dat )
{
	portInit();
	enterCritical();
	_TP_BusOutFastDrv( DT_EVTTRG_FAST_BIT, 2 );
	_TP_EventOutDrv( dat );
	exitCritical();
}

// =============================================================================
